@@ -13,21 +13,32 @@
  @todo        rewrite to api
 """
 
-from urllib import urlencode
 from lxml.html import fromstring
-from requests import get
 from json import loads
 from searx.engines.xpath import extract_text
+from searx.poolrequests import get
+from searx.url_utils import urlencode
+from searx.utils import match_language
 
 # engine dependent config
 categories = ['general']
 paging = True
 language_support = True
-supported_languages_url = 'https://duckduckgo.com/d2030.js'
+supported_languages_url = 'https://duckduckgo.com/util/u172.js'
 time_range_support = True
 
+language_aliases = {
+    'ar-SA': 'ar-XA',
+    'es-419': 'es-XL',
+    'ja': 'jp-JP',
+    'ko': 'kr-KR',
+    'sl-SI': 'sl-SL',
+    'zh-TW': 'tzh-TW',
+    'zh-HK': 'tzh-HK'
+}
+
 # search-url
-url = 'https://duckduckgo.com/html?{query}&s={offset}'
+url = 'https://duckduckgo.com/html?{query}&s={offset}&dc={dc_param}'
 time_range_url = '&df={range}'
 
 time_range_dict = {'day': 'd',
@@ -41,48 +52,31 @@ title_xpath = './/a[@class="result__a"]'
 content_xpath = './/a[@class="result__snippet"]'
 
 
-# do search-request
+# match query's language to a region code that duckduckgo will accept
+def get_region_code(lang, lang_list=[]):
+    if lang == 'all':
+        return None
+
+    lang_code = match_language(lang, lang_list, language_aliases, 'wt-WT')
+    lang_parts = lang_code.split('-')
+
+    # country code goes first
+    return lang_parts[1].lower() + '-' + lang_parts[0].lower()
+
+
 def request(query, params):
     if params['time_range'] and params['time_range'] not in time_range_dict:
         return params
 
     offset = (params['pageno'] - 1) * 30
 
-    # custom fixes for languages
-    if params['language'] == 'all':
-        locale = None
-    elif params['language'][:2] == 'ja':
-        locale = 'jp-jp'
-    elif params['language'][:2] == 'sl':
-        locale = 'sl-sl'
-    elif params['language'] == 'zh-TW':
-        locale = 'tw-tzh'
-    elif params['language'] == 'zh-HK':
-        locale = 'hk-tzh'
-    elif params['language'][-2:] == 'SA':
-        locale = 'xa-' + params['language'].split('-')[0]
-    elif params['language'][-2:] == 'GB':
-        locale = 'uk-' + params['language'].split('-')[0]
-    else:
-        locale = params['language'].split('-')
-        if len(locale) == 2:
-            # country code goes first
-            locale = locale[1].lower() + '-' + locale[0].lower()
-        else:
-            # tries to get a country code from language
-            locale = locale[0].lower()
-            for lc in supported_languages:
-                lc = lc.split('-')
-                if locale == lc[0]:
-                    locale = lc[1].lower() + '-' + lc[0].lower()
-                    break
-
-    if locale:
+    region_code = get_region_code(params['language'], supported_languages)
+    if region_code:
         params['url'] = url.format(
-            query=urlencode({'q': query, 'kl': locale}), offset=offset)
+            query=urlencode({'q': query, 'kl': region_code}), offset=offset, dc_param=offset)
     else:
         params['url'] = url.format(
-            query=urlencode({'q': query}), offset=offset)
+            query=urlencode({'q': query}), offset=offset, dc_param=offset)
 
     if params['time_range'] in time_range_dict:
         params['url'] += time_range_url.format(range=time_range_dict[params['time_range']])
@@ -129,4 +123,4 @@ def _fetch_supported_languages(resp):
     regions_json = loads(response_page)
     supported_languages = map((lambda x: x[3:] + '-' + x[:2].upper()), regions_json.keys())
 
-    return supported_languages
+    return list(supported_languages)
