@@ -22,11 +22,12 @@ if __name__ == '__main__':
     from os.path import realpath, dirname
     path.append(realpath(dirname(realpath(__file__)) + '/../'))
 
-import cStringIO
 import hashlib
 import hmac
 import json
 import os
+import sys
+
 import requests
 
 from searx import logger
@@ -42,8 +43,6 @@ except:
     exit(1)
 from cgi import escape
 from datetime import datetime, timedelta
-from urllib import urlencode
-from urlparse import urlparse, urljoin
 from werkzeug.contrib.fixers import ProxyFix
 from flask import (
     Flask, request, render_template, url_for, Response, make_response,
@@ -52,48 +51,70 @@ from flask import (
 from flask_babel import Babel, gettext, format_date, format_decimal
 from flask.json import jsonify
 from searx import settings, searx_dir, searx_debug
+from searx.exceptions import SearxParameterException
 from searx.engines import (
     categories, engines, engine_shortcuts, get_engines_stats, initialize_engines
 )
 from searx.utils import (
-    UnicodeWriter, highlight_content, html_to_text, get_themes,
-    get_static_files, get_result_templates, gen_useragent, dict_subset,
-    prettify_url
+    UnicodeWriter, highlight_content, html_to_text, get_resources_directory,
+    get_static_files, get_result_templates, get_themes, gen_useragent,
+    dict_subset, prettify_url, match_language
 )
 from searx.version import VERSION_STRING
-from searx.languages import language_codes
+from searx.languages import language_codes as languages
 from searx.search import SearchWithPlugins, get_search_query_from_webapp
 from searx.query import RawTextQuery
 from searx.autocomplete import searx_bang, backends as autocomplete_backends
 from searx.plugins import plugins
-from searx.preferences import Preferences, ValidationException
+from searx.plugins.oa_doi_rewrite import get_doi_resolver
+from searx.preferences import Preferences, ValidationException, LANGUAGE_CODES
 from searx.answerers import answerers
+from searx.url_utils import urlencode, urlparse, urljoin
+from searx.utils import new_hmac
 
-# check if the pyopenssl, ndg-httpsclient, pyasn1 packages are installed.
-# They are needed for SSL connection without trouble, see #298
+# check if the pyopenssl package is installed.
+# It is needed for SSL connection without trouble, see #298
 try:
     import OpenSSL.SSL  # NOQA
-    import ndg.httpsclient  # NOQA
-    import pyasn1  # NOQA
 except ImportError:
-    logger.critical("The pyopenssl, ndg-httpsclient, pyasn1 packages have to be installed.\n"
+    logger.critical("The pyopenssl package has to be installed.\n"
                     "Some HTTPS connections will fail")
 
+try:
+    from cStringIO import StringIO
+except:
+    from io import StringIO
+
+
+if sys.version_info[0] == 3:
+    unicode = str
+    PY3 = True
+else:
+    PY3 = False
+
 # serve pages with HTTP/1.1
 from werkzeug.serving import WSGIRequestHandler
 WSGIRequestHandler.protocol_version = "HTTP/{}".format(settings['server'].get('http_protocol_version', '1.0'))
 
-static_path, templates_path, themes =\
-    get_themes(settings['ui']['themes_path']
-               if settings['ui']['themes_path']
-               else searx_dir)
+# about static
+static_path = get_resources_directory(searx_dir, 'static', settings['ui']['static_path'])
+logger.debug('static directory is %s', static_path)
+static_files = get_static_files(static_path)
 
+# about templates
 default_theme = settings['ui']['default_theme']
+templates_path = get_resources_directory(searx_dir, 'templates', settings['ui']['templates_path'])
+logger.debug('templates directory is %s', templates_path)
+themes = get_themes(templates_path)
+result_templates = get_result_templates(templates_path)
+global_favicons = []
+for indice, theme in enumerate(themes):
+    global_favicons.append([])
+    theme_img_path = os.path.join(static_path, 'themes', theme, 'img', 'icons')
+    for (dirpath, dirnames, filenames) in os.walk(theme_img_path):
+        global_favicons[indice].extend(filenames)
 
-static_files = get_static_files(searx_dir)
-
-result_templates = get_result_templates(searx_dir)
-
+# Flask app
 app = Flask(
     __name__,
     static_folder=static_path,
@@ -104,20 +125,15 @@ app.jinja_env.trim_blocks = True
 app.jinja_env.lstrip_blocks = True
 app.secret_key = settings['server']['secret_key']
 
-if not searx_debug or os.environ.get("WERKZEUG_RUN_MAIN") == "true":
+if not searx_debug \
+   or os.environ.get("WERKZEUG_RUN_MAIN") == "true" \
+   or os.environ.get('UWSGI_ORIGINAL_PROC_NAME') is not None:
     initialize_engines(settings['engines'])
 
 babel = Babel(app)
 
 rtl_locales = ['ar', 'arc', 'bcc', 'bqi', 'ckb', 'dv', 'fa', 'glk', 'he',
-               'ku', 'mzn', 'pnb'', ''ps', 'sd', 'ug', 'ur', 'yi']
-
-global_favicons = []
-for indice, theme in enumerate(themes):
-    global_favicons.append([])
-    theme_img_path = searx_dir + "/static/themes/" + theme + "/img/icons/"
-    for (dirpath, dirnames, filenames) in os.walk(theme_img_path):
-        global_favicons[indice].extend(filenames)
+               'ku', 'mzn', 'pnb', 'ps', 'sd', 'ug', 'ur', 'yi']
 
 # used when translating category names
 _category_names = (gettext('files'),
@@ -131,7 +147,7 @@ _category_names = (gettext('files'),
                    gettext('map'),
                    gettext('science'))
 
-outgoing_proxies = settings['outgoing'].get('proxies', None)
+outgoing_proxies = settings['outgoing'].get('proxies') or None
 
 
 @babel.localeselector
@@ -149,6 +165,9 @@ def get_locale():
        and request.form['locale'] in settings['locales']:
         locale = request.form['locale']
 
+    if locale == 'zh_TW':
+        locale = 'zh_Hant_TW'
+
     return locale
 
 
@@ -226,7 +245,7 @@ def get_current_theme_name(override=None):
     2. cookies
     3. settings"""
 
-    if override and override in themes:
+    if override and (override in themes or override == '__common__'):
         return override
     theme_name = request.args.get('theme', request.preferences.get_value('theme'))
     if theme_name not in themes:
@@ -279,7 +298,7 @@ def image_proxify(url):
     if settings.get('result_proxy'):
         return proxify(url)
 
-    h = hmac.new(settings['server']['secret_key'], url.encode('utf-8'), hashlib.sha256).hexdigest()
+    h = new_hmac(settings['server']['secret_key'], url.encode('utf-8'))
 
     return '{0}?{1}'.format(url_for('image_proxy'),
                             urlencode(dict(url=url.encode('utf-8'), h=h)))
@@ -333,16 +352,18 @@ def render(template_name, override_theme=None, **kwargs):
 
     kwargs['safesearch'] = str(request.preferences.get_value('safesearch'))
 
-    kwargs['language_codes'] = language_codes
+    kwargs['language_codes'] = languages
     if 'current_language' not in kwargs:
-        kwargs['current_language'] = request.preferences.get_value('language')
+        kwargs['current_language'] = match_language(request.preferences.get_value('language'),
+                                                    LANGUAGE_CODES,
+                                                    fallback=settings['search']['language'])
 
     # override url_for function in templates
     kwargs['url_for'] = url_for_theme
 
     kwargs['image_proxify'] = image_proxify
 
-    kwargs['proxify'] = proxify if settings.get('result_proxy') else None
+    kwargs['proxify'] = proxify if settings.get('result_proxy', {}).get('url') else None
 
     kwargs['get_result_template'] = get_result_template
 
@@ -358,6 +379,10 @@ def render(template_name, override_theme=None, **kwargs):
 
     kwargs['results_on_new_tab'] = request.preferences.get_value('results_on_new_tab')
 
+    kwargs['unicode'] = unicode
+
+    kwargs['preferences'] = request.preferences
+
     kwargs['scripts'] = set()
     for plugin in request.user_plugins:
         for script in plugin.js_dependencies:
@@ -376,10 +401,10 @@ def render(template_name, override_theme=None, **kwargs):
 def pre_request():
     request.errors = []
 
-    preferences = Preferences(themes, categories.keys(), engines, plugins)
+    preferences = Preferences(themes, list(categories.keys()), engines, plugins)
     request.preferences = preferences
     try:
-        preferences.parse_cookies(request.cookies)
+        preferences.parse_dict(request.cookies)
     except:
         request.errors.append(gettext('Invalid settings, please edit your preferences'))
 
@@ -390,6 +415,15 @@ def pre_request():
         if k not in request.form:
             request.form[k] = v
 
+    if request.form.get('preferences'):
+        preferences.parse_encoded_data(request.form['preferences'])
+    else:
+        try:
+            preferences.parse_dict(request.form)
+        except Exception as e:
+            logger.exception('invalid settings')
+            request.errors.append(gettext('Invalid settings'))
+
     # request.user_plugins
     request.user_plugins = []
     allowed_plugins = preferences.plugins.get_enabled()
@@ -400,6 +434,34 @@ def pre_request():
             request.user_plugins.append(plugin)
 
 
+def index_error(output_format, error_message):
+    if output_format == 'json':
+        return Response(json.dumps({'error': error_message}),
+                        mimetype='application/json')
+    elif output_format == 'csv':
+        response = Response('', mimetype='application/csv')
+        cont_disp = 'attachment;Filename=searx.csv'
+        response.headers.add('Content-Disposition', cont_disp)
+        return response
+    elif output_format == 'rss':
+        response_rss = render(
+            'opensearch_response_rss.xml',
+            results=[],
+            q=request.form['q'] if 'q' in request.form else '',
+            number_of_results=0,
+            base_url=get_base_url(),
+            error_message=error_message,
+            override_theme='__common__',
+        )
+        return Response(response_rss, mimetype='text/xml')
+    else:
+        # html
+        request.errors.append(gettext('search error'))
+        return render(
+            'index.html',
+        )
+
+
 @app.route('/search', methods=['GET', 'POST'])
 @app.route('/', methods=['GET', 'POST'])
 def index():
@@ -408,10 +470,19 @@ def index():
     Supported outputs: html, json, csv, rss.
     """
 
+    # output_format
+    output_format = request.form.get('format', 'html')
+    if output_format not in ['html', 'csv', 'json', 'rss']:
+        output_format = 'html'
+
+    # check if there is query
     if request.form.get('q') is None:
-        return render(
-            'index.html',
-        )
+        if output_format == 'html':
+            return render(
+                'index.html',
+            )
+        else:
+            return index_error(output_format, 'No query'), 400
 
     # search
     search_query = None
@@ -419,31 +490,33 @@ def index():
     try:
         search_query = get_search_query_from_webapp(request.preferences, request.form)
         # search = Search(search_query) #  without plugins
-        search = SearchWithPlugins(search_query, request)
+        search = SearchWithPlugins(search_query, request.user_plugins, request)
         result_container = search.search()
-    except:
-        request.errors.append(gettext('search error'))
+    except Exception as e:
+        # log exception
         logger.exception('search error')
-        return render(
-            'index.html',
-        )
 
+        # is it an invalid input parameter or something else ?
+        if (issubclass(e.__class__, SearxParameterException)):
+            return index_error(output_format, e.message), 400
+        else:
+            return index_error(output_format, gettext('search error')), 500
+
+    # results
     results = result_container.get_ordered_results()
+    number_of_results = result_container.results_number()
+    if number_of_results < result_container.results_length():
+        number_of_results = 0
 
     # UI
     advanced_search = request.form.get('advanced_search', None)
-    output_format = request.form.get('format', 'html')
-    if output_format not in ['html', 'csv', 'json', 'rss']:
-        output_format = 'html'
 
     # output
     for result in results:
         if output_format == 'html':
             if 'content' in result and result['content']:
-                result['content'] = highlight_content(escape(result['content'][:1024]),
-                                                      search_query.query.encode('utf-8'))
-            result['title'] = highlight_content(escape(result['title'] or u''),
-                                                search_query.query.encode('utf-8'))
+                result['content'] = highlight_content(escape(result['content'][:1024]), search_query.query)
+            result['title'] = highlight_content(escape(result['title'] or u''), search_query.query)
         else:
             if result.get('content'):
                 result['content'] = html_to_text(result['content']).strip()
@@ -470,20 +543,19 @@ def index():
                 else:
                     result['publishedDate'] = format_date(result['publishedDate'])
 
-    number_of_results = result_container.results_number()
-    if number_of_results < result_container.results_length():
-        number_of_results = 0
-
     if output_format == 'json':
-        return Response(json.dumps({'query': search_query.query,
+        return Response(json.dumps({'query': search_query.query.decode('utf-8'),
                                     'number_of_results': number_of_results,
                                     'results': results,
                                     'answers': list(result_container.answers),
+                                    'corrections': list(result_container.corrections),
                                     'infoboxes': result_container.infoboxes,
-                                    'suggestions': list(result_container.suggestions)}),
+                                    'suggestions': list(result_container.suggestions),
+                                    'unresponsive_engines': list(result_container.unresponsive_engines)},
+                                   default=lambda item: list(item) if isinstance(item, set) else item),
                         mimetype='application/json')
     elif output_format == 'csv':
-        csv = UnicodeWriter(cStringIO.StringIO())
+        csv = UnicodeWriter(StringIO())
         keys = ('title', 'url', 'content', 'host', 'engine', 'score')
         csv.writerow(keys)
         for row in results:
@@ -491,7 +563,7 @@ def index():
             csv.writerow([row.get(key, '') for key in keys])
         csv.stream.seek(0)
         response = Response(csv.stream.read(), mimetype='application/csv')
-        cont_disp = 'attachment;Filename=searx_-_{0}.csv'.format(search_query.query.encode('utf-8'))
+        cont_disp = 'attachment;Filename=searx_-_{0}.csv'.format(search_query.query)
         response.headers.add('Content-Disposition', cont_disp)
         return response
     elif output_format == 'rss':
@@ -500,7 +572,8 @@ def index():
             results=results,
             q=request.form['q'],
             number_of_results=number_of_results,
-            base_url=get_base_url()
+            base_url=get_base_url(),
+            override_theme='__common__',
         )
         return Response(response_rss, mimetype='text/xml')
 
@@ -515,9 +588,13 @@ def index():
         advanced_search=advanced_search,
         suggestions=result_container.suggestions,
         answers=result_container.answers,
+        corrections=result_container.corrections,
         infoboxes=result_container.infoboxes,
         paging=result_container.paging,
-        current_language=search_query.lang,
+        unresponsive_engines=result_container.unresponsive_engines,
+        current_language=match_language(search_query.lang,
+                                        LANGUAGE_CODES,
+                                        fallback=settings['search']['language']),
         base_url=get_base_url(),
         theme=get_current_theme_name(),
         favicons=global_favicons[themes.index(get_current_theme_name())]
@@ -540,7 +617,10 @@ def autocompleter():
     disabled_engines = request.preferences.engines.get_disabled()
 
     # parse query
-    raw_text_query = RawTextQuery(request.form.get('q', '').encode('utf-8'), disabled_engines)
+    if PY3:
+        raw_text_query = RawTextQuery(request.form.get('q', b''), disabled_engines)
+    else:
+        raw_text_query = RawTextQuery(request.form.get('q', u'').encode('utf-8'), disabled_engines)
     raw_text_query.parse_query()
 
     # check if search query is set
@@ -611,6 +691,7 @@ def preferences():
                              'warn_time': False}
             if e.timeout > settings['outgoing']['request_timeout']:
                 stats[e.name]['warn_timeout'] = True
+            stats[e.name]['supports_selected_language'] = _is_selected_language_supported(e, request.preferences)
 
     # get first element [0], the engine time,
     # and then the second element [1] : the time (the first one is the label)
@@ -632,11 +713,23 @@ def preferences():
                   shortcuts={y: x for x, y in engine_shortcuts.items()},
                   themes=themes,
                   plugins=plugins,
+                  doi_resolvers=settings['doi_resolvers'],
+                  current_doi_resolver=get_doi_resolver(request.args, request.preferences.get_value('doi_resolver')),
                   allowed_plugins=allowed_plugins,
                   theme=get_current_theme_name(),
+                  preferences_url_params=request.preferences.get_as_url_params(),
+                  base_url=get_base_url(),
                   preferences=True)
 
 
+def _is_selected_language_supported(engine, preferences):
+    language = preferences.get_value('language')
+    return (language == 'all'
+            or match_language(language,
+                              getattr(engine, 'supported_languages', []),
+                              getattr(engine, 'language_aliases', {}), None))
+
+
 @app.route('/image_proxy', methods=['GET'])
 def image_proxy():
     url = request.args.get('url').encode('utf-8')
@@ -644,7 +737,7 @@ def image_proxy():
     if not url:
         return '', 400
 
-    h = hmac.new(settings['server']['secret_key'], url, hashlib.sha256).hexdigest()
+    h = new_hmac(settings['server']['secret_key'], url)
 
     if h != request.args.get('h'):
         return '', 400
@@ -671,7 +764,7 @@ def image_proxy():
         logger.debug('image-proxy: wrong content-type: {0}'.format(resp.headers.get('content-type')))
         return '', 400
 
-    img = ''
+    img = b''
     chunk_counter = 0
 
     for chunk in resp.iter_content(1024 * 1024):
@@ -720,7 +813,8 @@ def opensearch():
     ret = render('opensearch.xml',
                  opensearch_method=method,
                  host=get_base_url(),
-                 urljoin=urljoin)
+                 urljoin=urljoin,
+                 override_theme='__common__')
 
     resp = Response(response=ret,
                     status=200,
@@ -731,7 +825,8 @@ def opensearch():
 @app.route('/favicon.ico')
 def favicon():
     return send_from_directory(os.path.join(app.root_path,
-                                            'static/themes',
+                                            static_path,
+                                            'themes',
                                             get_current_theme_name(),
                                             'img'),
                                'favicon.png',
@@ -772,7 +867,10 @@ def config():
                     'autocomplete': settings['search']['autocomplete'],
                     'safe_search': settings['search']['safe_search'],
                     'default_theme': settings['ui']['default_theme'],
-                    'version': VERSION_STRING})
+                    'version': VERSION_STRING,
+                    'doi_resolvers': [r for r in settings['doi_resolvers']],
+                    'default_doi_resolver': settings['default_doi_resolver'],
+                    })
 
 
 @app.errorhandler(404)
@@ -781,6 +879,7 @@ def page_not_found(e):
 
 
 def run():
+    logger.debug('starting webserver on %s:%s', settings['server']['port'], settings['server']['bind_address'])
     app.run(
         debug=searx_debug,
         use_debugger=searx_debug,
